@@ -19,17 +19,17 @@
  * Boston, MA 02110-1301, USA.
  */
 /**
- * SECTION:element-latencytracer
+ * SECTION:tracer-latency
  * @short_description: log processing latency stats
  *
  * A tracing module that determines src-to-sink latencies by injecting custom
  * events at sources and process them at sinks. This elements supports tracing
  * the entire pipeline latency and per element latency. By default, only
  * pipeline latency is traced. The 'flags' parameter can be used to enabled
- * element tracing.
+ * element tracing and/or the latency reported by each element.
  *
  * ```
- * GST_TRACERS="latency(flags=pipeline+element)" GST_DEBUG=GST_TRACER:7 ./...
+ * GST_TRACERS="latency(flags=pipeline+element+reported)" GST_DEBUG=GST_TRACER:7 ./...
  * ```
  */
 /* TODO(ensonic): if there are two sources feeding into a mixer/muxer and later
@@ -59,10 +59,6 @@ static void latency_query_stack_destroy (gpointer data);
 
 static GQuark latency_probe_id;
 static GQuark sub_latency_probe_id;
-static GQuark latency_probe_pad;
-static GQuark latency_probe_element;
-static GQuark latency_probe_element_id;
-static GQuark latency_probe_ts;
 static GQuark drop_sub_latency_quark;
 
 static GstTracerRecord *tr_latency;
@@ -105,12 +101,15 @@ get_real_pad_parent (GstPad * pad)
   if (!pad)
     return NULL;
 
-  parent = GST_OBJECT_PARENT (pad);
+  parent = gst_object_get_parent (GST_OBJECT_CAST (pad));
 
   /* if parent of pad is a ghost-pad, then pad is a proxy_pad */
   if (parent && GST_IS_GHOST_PAD (parent)) {
+    GstObject *tmp;
     pad = GST_PAD_CAST (parent);
-    parent = GST_OBJECT_PARENT (pad);
+    tmp = gst_object_get_parent (GST_OBJECT_CAST (pad));
+    gst_object_unref (parent);
+    parent = tmp;
   }
   return GST_ELEMENT_CAST (parent);
 }
@@ -176,16 +175,19 @@ log_latency (const GstStructure * data, GstElement * sink_parent,
   const GValue *value;
   gchar *sink, *element_sink, *id_element_sink;
 
-  value = gst_structure_id_get_value (data, latency_probe_ts);
+  g_return_if_fail (sink_parent);
+  g_return_if_fail (sink_pad);
+
+  value = gst_structure_get_value (data, "latency_probe.ts");
   src_ts = g_value_get_uint64 (value);
 
-  value = gst_structure_id_get_value (data, latency_probe_pad);
+  value = gst_structure_get_value (data, "latency_probe.pad");
   src = g_value_get_string (value);
 
-  value = gst_structure_id_get_value (data, latency_probe_element);
+  value = gst_structure_get_value (data, "latency_probe.element");
   element_src = g_value_get_string (value);
 
-  value = gst_structure_id_get_value (data, latency_probe_element_id);
+  value = gst_structure_get_value (data, "latency_probe.element_id");
   id_element_src = g_value_get_string (value);
 
   id_element_sink = g_strdup_printf ("%p", sink_parent);
@@ -207,13 +209,16 @@ log_element_latency (const GstStructure * data, GstElement * parent,
   gchar *pad_name, *element_name, *element_id;
   const GValue *value;
 
+  g_return_if_fail (parent);
+  g_return_if_fail (pad);
+
   element_id = g_strdup_printf ("%p", parent);
   element_name = gst_element_get_name (parent);
   pad_name = gst_pad_get_name (pad);
 
   /* TODO filtering */
 
-  value = gst_structure_id_get_value (data, latency_probe_ts);
+  value = gst_structure_get_value (data, "latency_probe.ts");
   src_ts = g_value_get_uint64 (value);
 
   gst_tracer_record_log (tr_element_latency, element_id, element_name, pad_name,
@@ -228,7 +233,7 @@ static void
 send_latency_probe (GstLatencyTracer * self, GstElement * parent, GstPad * pad,
     guint64 ts)
 {
-  GstPad *peer_pad = GST_PAD_PEER (pad);
+  GstPad *peer_pad = gst_pad_get_peer (pad);
   GstElement *peer_parent = get_real_pad_parent (peer_pad);
 
   /* allow for non-parented pads to send latency probes as used in e.g.
@@ -237,18 +242,18 @@ send_latency_probe (GstLatencyTracer * self, GstElement * parent, GstPad * pad,
     gchar *pad_name, *element_name, *element_id;
     GstEvent *latency_probe;
 
-    if (self->flags & GST_LATENCY_TRACER_FLAG_PIPELINE &&
+    if (parent && self->flags & GST_LATENCY_TRACER_FLAG_PIPELINE &&
         GST_OBJECT_FLAG_IS_SET (parent, GST_ELEMENT_FLAG_SOURCE)) {
       element_id = g_strdup_printf ("%p", parent);
       element_name = gst_element_get_name (parent);
       pad_name = gst_pad_get_name (pad);
 
       latency_probe = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM,
-          gst_structure_new_id (latency_probe_id,
-              latency_probe_element_id, G_TYPE_STRING, element_id,
-              latency_probe_element, G_TYPE_STRING, element_name,
-              latency_probe_pad, G_TYPE_STRING, pad_name,
-              latency_probe_ts, G_TYPE_UINT64, ts, NULL));
+          gst_structure_new_static_str ("latency_probe.id",
+              "latency_probe.element_id", G_TYPE_STRING, element_id,
+              "latency_probe.element", G_TYPE_STRING, element_name,
+              "latency_probe.pad", G_TYPE_STRING, pad_name,
+              "latency_probe.ts", G_TYPE_UINT64, ts, NULL));
 
       GST_DEBUG ("%s_%s: Sending latency event %p", GST_DEBUG_PAD_NAME (pad),
           latency_probe);
@@ -259,17 +264,18 @@ send_latency_probe (GstLatencyTracer * self, GstElement * parent, GstPad * pad,
       gst_pad_push_event (pad, latency_probe);
     }
 
-    if (self->flags & GST_LATENCY_TRACER_FLAG_ELEMENT) {
+    if (peer_parent && peer_pad &&
+        self->flags & GST_LATENCY_TRACER_FLAG_ELEMENT) {
       element_id = g_strdup_printf ("%p", peer_parent);
       element_name = gst_element_get_name (peer_parent);
       pad_name = gst_pad_get_name (peer_pad);
 
       latency_probe = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM,
-          gst_structure_new_id (sub_latency_probe_id,
-              latency_probe_element_id, G_TYPE_STRING, element_id,
-              latency_probe_element, G_TYPE_STRING, element_name,
-              latency_probe_pad, G_TYPE_STRING, pad_name,
-              latency_probe_ts, G_TYPE_UINT64, ts, NULL));
+          gst_structure_new_static_str ("sub_latency_probe.id",
+              "latency_probe.element_id", G_TYPE_STRING, element_id,
+              "latency_probe.element", G_TYPE_STRING, element_name,
+              "latency_probe.pad", G_TYPE_STRING, pad_name,
+              "latency_probe.ts", G_TYPE_UINT64, ts, NULL));
 
       GST_DEBUG ("%s_%s: Sending sub-latency event %p",
           GST_DEBUG_PAD_NAME (pad), latency_probe);
@@ -280,36 +286,44 @@ send_latency_probe (GstLatencyTracer * self, GstElement * parent, GstPad * pad,
       g_free (element_id);
     }
   }
+  if (peer_pad)
+    gst_object_unref (peer_pad);
+  if (peer_parent)
+    gst_object_unref (peer_parent);
 }
 
 static void
 calculate_latency (GstElement * parent, GstPad * pad, guint64 ts)
 {
-  GstElement *peer_parent = get_real_pad_parent (GST_PAD_PEER (pad));
-
   if (parent && (!GST_IS_BIN (parent)) &&
       (!GST_OBJECT_FLAG_IS_SET (parent, GST_ELEMENT_FLAG_SOURCE))) {
     GstEvent *ev;
+    GstPad *peer_pad = gst_pad_get_peer (pad);
+    GstElement *peer_parent = get_real_pad_parent (peer_pad);
 
-    /* FIXME unsafe use of peer */
-    if (GST_OBJECT_FLAG_IS_SET (peer_parent, GST_ELEMENT_FLAG_SINK)) {
+    /* Protect against element being unlinked */
+    if (peer_pad && peer_parent &&
+        GST_OBJECT_FLAG_IS_SET (peer_parent, GST_ELEMENT_FLAG_SINK)) {
       ev = g_object_get_qdata ((GObject *) pad, latency_probe_id);
-      GST_DEBUG ("%s_%s: Should log full lantency now (event %p)",
+      GST_DEBUG ("%s_%s: Should log full latency now (event %p)",
           GST_DEBUG_PAD_NAME (pad), ev);
       if (ev) {
-        log_latency (gst_event_get_structure (ev), peer_parent,
-            GST_PAD_PEER (pad), ts);
+        log_latency (gst_event_get_structure (ev), peer_parent, peer_pad, ts);
         g_object_set_qdata ((GObject *) pad, latency_probe_id, NULL);
       }
     }
 
     ev = g_object_get_qdata ((GObject *) pad, sub_latency_probe_id);
-    GST_DEBUG ("%s_%s: Should log sub lantency now (event %p)",
+    GST_DEBUG ("%s_%s: Should log sub latency now (event %p)",
         GST_DEBUG_PAD_NAME (pad), ev);
     if (ev) {
       log_element_latency (gst_event_get_structure (ev), parent, pad, ts);
       g_object_set_qdata ((GObject *) pad, sub_latency_probe_id, NULL);
     }
+    if (peer_pad)
+      gst_object_unref (peer_pad);
+    if (peer_parent)
+      gst_object_unref (peer_parent);
   }
 }
 
@@ -321,6 +335,9 @@ do_push_buffer_pre (GstTracer * tracer, guint64 ts, GstPad * pad)
 
   send_latency_probe (self, parent, pad, ts);
   calculate_latency (parent, pad, ts);
+
+  if (parent)
+    gst_object_unref (parent);
 }
 
 static void
@@ -331,6 +348,9 @@ do_pull_range_pre (GstTracer * tracer, guint64 ts, GstPad * pad)
   GstElement *parent = get_real_pad_parent (peer_pad);
 
   send_latency_probe (self, parent, peer_pad, ts);
+
+  if (parent)
+    gst_object_unref (parent);
 }
 
 static void
@@ -339,6 +359,9 @@ do_pull_range_post (GstTracer * self, guint64 ts, GstPad * pad)
   GstElement *parent = get_real_pad_parent (pad);
 
   calculate_latency (parent, pad, ts);
+
+  if (parent)
+    gst_object_unref (parent);
 }
 
 static GstPadProbeReturn
@@ -351,22 +374,22 @@ do_drop_sub_latency_event (GstPad * pad, GstPadProbeInfo * info,
   if (GST_EVENT_TYPE (ev) == GST_EVENT_CUSTOM_DOWNSTREAM) {
     const GstStructure *data = gst_event_get_structure (ev);
 
-    if (gst_structure_get_name_id (data) == sub_latency_probe_id) {
-      /* FIXME unsafe peer pad usage */
-      GstPad *peer_pad = GST_PAD_PEER (pad);
+    if (gst_structure_has_name (data, "sub_latency_probe.id")) {
+      GstPad *peer_pad = gst_pad_get_peer (pad);
       GstElement *peer_parent = get_real_pad_parent (peer_pad);
       const GValue *value;
       gchar *element_id = g_strdup_printf ("%p", peer_parent);
-      gchar *pad_name = gst_pad_get_name (peer_pad);
+      gchar *pad_name = peer_pad ? gst_pad_get_name (peer_pad) : NULL;
       const gchar *value_element_id, *value_pad_name;
 
       /* Get the element id, element name and pad name from data */
-      value = gst_structure_id_get_value (data, latency_probe_element_id);
+      value = gst_structure_get_value (data, "latency_probe.element_id");
       value_element_id = g_value_get_string (value);
-      value = gst_structure_id_get_value (data, latency_probe_pad);
+      value = gst_structure_get_value (data, "latency_probe.pad");
       value_pad_name = g_value_get_string (value);
 
-      if (!g_str_equal (value_element_id, element_id) ||
+      if (pad_name == NULL ||
+          !g_str_equal (value_element_id, element_id) ||
           !g_str_equal (value_pad_name, pad_name)) {
         GST_DEBUG ("%s_%s: Dropping sub-latency event",
             GST_DEBUG_PAD_NAME (pad));
@@ -375,6 +398,11 @@ do_drop_sub_latency_event (GstPad * pad, GstPadProbeInfo * info,
 
       g_free (pad_name);
       g_free (element_id);
+
+      if (peer_pad)
+        gst_object_unref (peer_pad);
+      if (peer_parent)
+        gst_object_unref (peer_parent);
     }
   }
 
@@ -385,18 +413,17 @@ static void
 do_push_event_pre (GstTracer * self, guint64 ts, GstPad * pad, GstEvent * ev)
 {
   GstElement *parent = get_real_pad_parent (pad);
-  GstPad *peer_pad = GST_PAD_PEER (pad);
-  GstElement *peer_parent = get_real_pad_parent (peer_pad);
 
   if (parent && (!GST_IS_BIN (parent)) &&
       (!GST_OBJECT_FLAG_IS_SET (parent, GST_ELEMENT_FLAG_SOURCE)) &&
       GST_EVENT_TYPE (ev) == GST_EVENT_CUSTOM_DOWNSTREAM) {
     const GstStructure *data = gst_event_get_structure (ev);
+    GstPad *peer_pad = gst_pad_get_peer (pad);
+    GstElement *peer_parent = get_real_pad_parent (peer_pad);
 
     /* if not set yet, add a pad probe that prevents sub-latency event from
      * flowing further */
-    if (gst_structure_get_name_id (data) == latency_probe_id) {
-
+    if (gst_structure_has_name (data, "latency_probe.id")) {
       if (!g_object_get_qdata ((GObject *) pad, drop_sub_latency_quark)) {
         GST_DEBUG ("%s_%s: Adding pad probe to drop sub-latency event",
             GST_DEBUG_PAD_NAME (pad));
@@ -406,8 +433,8 @@ do_push_event_pre (GstTracer * self, guint64 ts, GstPad * pad, GstEvent * ev)
             (gpointer) 1);
       }
 
-      /* FIXME unsafe peer access */
-      if (GST_OBJECT_FLAG_IS_SET (peer_parent, GST_ELEMENT_FLAG_SINK)) {
+      if (peer_parent == NULL
+          || GST_OBJECT_FLAG_IS_SET (peer_parent, GST_ELEMENT_FLAG_SINK)) {
         /* store event so that we can calculate latency when the buffer that
          * follows has been processed */
         g_object_set_qdata_full ((GObject *) pad, latency_probe_id,
@@ -415,20 +442,20 @@ do_push_event_pre (GstTracer * self, guint64 ts, GstPad * pad, GstEvent * ev)
       }
     }
 
-    if (gst_structure_get_name_id (data) == sub_latency_probe_id) {
+    if (gst_structure_has_name (data, "sub_latency_probe.id")) {
       const GValue *value;
       gchar *element_id = g_strdup_printf ("%p", peer_parent);
-      gchar *pad_name = gst_pad_get_name (peer_pad);
+      gchar *pad_name = peer_pad ? gst_pad_get_name (peer_pad) : NULL;
       const gchar *value_element_id, *value_pad_name;
 
       /* Get the element id, element name and pad name from data */
-      value = gst_structure_id_get_value (data, latency_probe_element_id);
+      value = gst_structure_get_value (data, "latency_probe.element_id");
       value_element_id = g_value_get_string (value);
-      value = gst_structure_id_get_value (data, latency_probe_pad);
+      value = gst_structure_get_value (data, "latency_probe.pad");
       value_pad_name = g_value_get_string (value);
 
       if (!g_str_equal (value_element_id, element_id) ||
-          !g_str_equal (value_pad_name, pad_name)) {
+          g_strcmp0 (value_pad_name, pad_name) != 0) {
         GST_DEBUG ("%s_%s: Storing sub-latency event",
             GST_DEBUG_PAD_NAME (pad));
         g_object_set_qdata_full ((GObject *) pad, sub_latency_probe_id,
@@ -438,7 +465,13 @@ do_push_event_pre (GstTracer * self, guint64 ts, GstPad * pad, GstEvent * ev)
       g_free (pad_name);
       g_free (element_id);
     }
+    if (peer_pad)
+      gst_object_unref (peer_pad);
+    if (peer_parent)
+      gst_object_unref (peer_parent);
   }
+  if (parent)
+    gst_object_unref (parent);
 }
 
 static void
@@ -453,7 +486,16 @@ do_query_post (GstLatencyTracer * tracer, GstClockTime ts, GstPad * pad,
     gchar *element_name, *element_id;
     struct LatencyQueryTableValue *value;
     GstElement *element = get_real_pad_parent (pad);
-    GstElement *peer_element = get_real_pad_parent (GST_PAD_PEER (pad));
+    GstPad *peer_pad = gst_pad_get_peer (pad);
+    GstElement *peer_element = get_real_pad_parent (peer_pad);
+
+    /* If something is being removed/unlinked, cleanup the stack so we can
+     * ignore this query in the trace. */
+    if (!element || !peer_element || !peer_pad) {
+      while ((value = local_latency_query_stack_pop ()))
+        latency_query_table_value_destroy (value);
+      return;
+    }
 
     /* Parse the query */
     gst_query_parse_latency (query, &live, &min, &max);
@@ -463,8 +505,11 @@ do_query_post (GstLatencyTracer * tracer, GstClockTime ts, GstPad * pad,
     while (value && value->peer_element == element) {
       min_prev = MAX (value->min, min_prev);
       max_prev = MAX (value->max, max_prev);
+      latency_query_table_value_destroy (value);
       value = local_latency_query_stack_pop ();
     }
+    if (value)
+      latency_query_table_value_destroy (value);
 
     /* Push to stack */
     value = g_new0 (struct LatencyQueryTableValue, 1);
@@ -485,54 +530,77 @@ do_query_post (GstLatencyTracer * tracer, GstClockTime ts, GstPad * pad,
     /* Clean up */
     g_free (element_name);
     g_free (element_id);
+
+    gst_object_unref (peer_pad);
+    gst_object_unref (peer_element);
+    gst_object_unref (element);
   }
 }
 
 /* tracer class */
 
+/* Define the GType for GstLatencyTracerFlags */
+static GType
+gst_latency_tracer_flags_get_type (void)
+{
+  static GType type = 0;
+  static const GFlagsValue values[] = {
+    {GST_LATENCY_TRACER_FLAG_PIPELINE, "Trace pipeline latency", "pipeline"},
+    {GST_LATENCY_TRACER_FLAG_ELEMENT, "Trace per-element latency", "element"},
+    {GST_LATENCY_TRACER_FLAG_REPORTED_ELEMENT, "Trace reported element latency",
+        "reported"},
+    {0, NULL, NULL}
+  };
+
+  if (!type) {
+    type = g_flags_register_static ("GstLatencyTracerFlags", values);
+  }
+  return type;
+}
+
+enum
+{
+  PROP_0,
+  PROP_FLAGS,
+  PROP_LAST
+};
+
+static void
+gst_latency_tracer_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstLatencyTracer *self = GST_LATENCY_TRACER (object);
+
+  switch (prop_id) {
+    case PROP_FLAGS:
+      g_value_set_flags (value, self->flags);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_latency_tracer_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstLatencyTracer *self = GST_LATENCY_TRACER (object);
+
+  switch (prop_id) {
+    case PROP_FLAGS:
+      self->flags = g_value_get_flags (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static void
 gst_latency_tracer_constructed (GObject * object)
 {
-  GstLatencyTracer *self = GST_LATENCY_TRACER (object);
-  gchar *params, *tmp;
-  GstStructure *params_struct = NULL;
-
-  g_object_get (self, "params", &params, NULL);
-
-  if (!params)
-    return;
-
-  tmp = g_strdup_printf ("latency,%s", params);
-  params_struct = gst_structure_from_string (tmp, NULL);
-  g_free (tmp);
-
-  /* Read the flags if available */
-  if (params_struct) {
-    const gchar *flags = gst_structure_get_string (params_struct, "flags");
-
-    self->flags = 0;
-
-    if (flags) {
-      GStrv split = g_strsplit (flags, "+", -1);
-      gint i;
-
-      for (i = 0; split[i]; i++) {
-        if (g_str_equal (split[i], "pipeline"))
-          self->flags |= GST_LATENCY_TRACER_FLAG_PIPELINE;
-        else if (g_str_equal (split[i], "element"))
-          self->flags |= GST_LATENCY_TRACER_FLAG_ELEMENT;
-        else if (g_str_equal (split[i], "reported"))
-          self->flags |= GST_LATENCY_TRACER_FLAG_REPORTED_ELEMENT;
-        else
-          GST_WARNING ("Invalid latency tracer flags %s", split[i]);
-      }
-
-      g_strfreev (split);
-    }
-    gst_structure_free (params_struct);
-  }
-
-  g_free (params);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 }
 
 static void
@@ -540,57 +608,63 @@ gst_latency_tracer_class_init (GstLatencyTracerClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gst_tracer_class_set_use_structure_params (GST_TRACER_CLASS (klass), TRUE);
+
   gobject_class->constructed = gst_latency_tracer_constructed;
+  gobject_class->get_property = gst_latency_tracer_get_property;
+  gobject_class->set_property = gst_latency_tracer_set_property;
+
+  g_object_class_install_property (gobject_class, PROP_FLAGS,
+      g_param_spec_flags ("flags", "Flags",
+          "Flags to control what latency measurements to perform",
+          gst_latency_tracer_flags_get_type (),
+          GST_LATENCY_TRACER_FLAG_PIPELINE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
   latency_probe_id = g_quark_from_static_string ("latency_probe.id");
   sub_latency_probe_id = g_quark_from_static_string ("sub_latency_probe.id");
-  latency_probe_pad = g_quark_from_static_string ("latency_probe.pad");
-  latency_probe_element = g_quark_from_static_string ("latency_probe.element");
-  latency_probe_element_id =
-      g_quark_from_static_string ("latency_probe.element_id");
-  latency_probe_ts = g_quark_from_static_string ("latency_probe.ts");
   drop_sub_latency_quark =
       g_quark_from_static_string ("drop_sub_latency.quark");
 
   /* announce trace formats */
   /* *INDENT-OFF* */
   tr_latency = gst_tracer_record_new ("latency.class",
-      "src-element-id", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "src-element-id", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "src-element", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "src-element", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "src", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "src", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE, GST_TRACER_VALUE_SCOPE_PAD,
           NULL),
-      "sink-element-id", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "sink-element-id", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "sink-element", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "sink-element", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "sink", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "sink", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE, GST_TRACER_VALUE_SCOPE_PAD,
           NULL),
-      "time", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "time", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING,
               "time it took for the buffer to go from src to sink ns",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
           "max", G_TYPE_UINT64, G_MAXUINT64,
           NULL),
-      "ts", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "ts", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING, "ts when the latency has been logged",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
@@ -599,28 +673,28 @@ gst_latency_tracer_class_init (GstLatencyTracerClass * klass)
       NULL);
 
   tr_element_latency = gst_tracer_record_new ("element-latency.class",
-      "element-id", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "element-id", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "element", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "element", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "src", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "src", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE, GST_TRACER_VALUE_SCOPE_PAD,
           NULL),
-      "time", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "time", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING,
               "time it took for the buffer to go from src to sink ns",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
           "max", G_TYPE_UINT64, G_MAXUINT64,
           NULL),
-      "ts", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "ts", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING, "ts when the latency has been logged",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
@@ -631,35 +705,35 @@ gst_latency_tracer_class_init (GstLatencyTracerClass * klass)
 
   tr_element_reported_latency = gst_tracer_record_new (
       "element-reported-latency.class",
-      "element-id", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "element-id", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "element", GST_TYPE_STRUCTURE, gst_structure_new ("scope",
+      "element", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("scope",
           "type", G_TYPE_GTYPE, G_TYPE_STRING,
           "related-to", GST_TYPE_TRACER_VALUE_SCOPE,
           GST_TRACER_VALUE_SCOPE_ELEMENT,
           NULL),
-      "live", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "live", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_BOOLEAN,
           "description", G_TYPE_STRING,
               "wether the it is a live stream or not",
           NULL),
-      "min", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "min", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING,
               "the minimum reported latency",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
           "max", G_TYPE_UINT64, G_MAXUINT64,
           NULL),
-      "max", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "max", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING, "the maximum reported latency",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
           "max", G_TYPE_UINT64, G_MAXUINT64,
           NULL),
-      "ts", GST_TYPE_STRUCTURE, gst_structure_new ("value",
+      "ts", GST_TYPE_STRUCTURE, gst_structure_new_static_str ("value",
           "type", G_TYPE_GTYPE, G_TYPE_UINT64,
           "description", G_TYPE_STRING, "ts when the latency has been reported",
           "min", G_TYPE_UINT64, G_GUINT64_CONSTANT (0),
